/*
 * The Program materials contained in this file are IBM copyright materials.
 * COPYRIGHT International Business Machines Corp. 2015
 * All Rights Reserved * Licensed Materials - Property of IBM
 * US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

var serverExplorer = (function() {
	
	var cachedServerList = null;
	var cachedFilteredServerList = null;
	var filterApplied = false;
	
	var renderServerExplorer = function() {
		var deferred = new $.Deferred();

		retrieveCollectiveServerList().done(function(serverList) {
			
			// Initialize filtered list with complete set of servers
			cachedFilteredServerList = cachedServerList;
		
			// Initially sort servers by name
			handleSort("name");
			
			renderServerList();
			
			deferred.resolve();
		}).fail(function() {
			deferred.reject();
		});
		
		// Clear filter
		var serverExplorerSearchInput = $("#serverExplorerSearchInput");
		serverExplorerSearchInput.val("");
		$("#serverExplorerClearButton").addClass("hidden");
		$("#serverExplorerSearchButton").attr("disabled", "disabled");
		
		// Handle bidi
		if(globalization.isBidiEnabled()) {
			var dirValue = globalization.getBidiTextDirection();
			if(dirValue != "contextual") {
				serverExplorerSearchInput.attr("dir", dirValue);
			}
		}
		
		return deferred;
	};
	
	
	var retrieveCollectiveServerList = function() {
		var deferred = new $.Deferred();
		if(cachedServerList != null) {
			deferred.resolve();
		} else {
			collectiveUtils.retrieveCollectiveServerList().done(function(serverList) {
				cachedServerList = serverList;
				deferred.resolve();
			}).fail(function() {
				deferred.reject();
			});
		}
		
		return deferred;
	};
	
	
	var renderServerList = function() {

		var serverExplorerTableBody = $("#serverExplorerTableBody");
		serverExplorerTableBody.empty();
		
		var serverCount = cachedFilteredServerList.length;
		var title = "";
		
		if(serverCount == 1) {
			title = editorMessages.ONE_SERVER;
		} else {
			title = stringUtils.formatString(editorMessages.SERVERS, [serverCount]);
		}
		
		if(serverCount > 500) {			
			core.showControlById("serverExplorerTableTitleLimit");	
		} else {
			core.hideControlById("serverExplorerTableTitleLimit");
		}

		$("#serverExplorerTableTitleText").text(title);
				
		for(var i = 0; i < serverCount && i < 500; i++) {
			createServerEntry(cachedFilteredServerList[i], serverExplorerTableBody, $("#serverExplorerSearchInput").val());
		}
	};
	
	
	var applySearchFilter = function() {

		// Obtain search string
		var searchString = $("#serverExplorerSearchInput").val();
		
		// Only apply search string when provided
		if(searchString.length > 0) {
			
			// Update filter flag
			filterApplied = true;
			
			// Initialize list
			cachedFilteredServerList = [];
		
			// Apply filter string
			for(var i = 0; i < cachedServerList.length; i++) {
				var currentServer = cachedServerList[i];
				if(currentServer.name.indexOf(searchString) != -1 || (currentServer.cluster && currentServer.cluster.indexOf(searchString) != -1)
						|| currentServer.host.indexOf(searchString) != -1 || currentServer.wlpUserDir.indexOf(searchString) != -1) {
					cachedFilteredServerList.push(currentServer);
				}
			}
		} else {

			// Update filter flag
			filterApplied = false;

			// Unfiltered list if no search string is provided
			cachedFilteredServerList = cachedServerList;
			
		}
	};
	
	
	var createServerEntry = function(server, serverExplorerTableBody, searchFilter) {
		var cluster = server.cluster;
		
		// Cluster might be null
		if(!cluster) {
			cluster = "";
		}
		
		// Handle bidi
		var bidiDirMarkupServerName = "";
		var bidiDirMarkupCluster = "";
		var bidiDirMarkupHost = "";
		var bidiDirMarkupUserDir = "";
		if(globalization.isBidiEnabled()) {
			var dirValue = globalization.getBidiTextDirection();
			if(dirValue == "contextual") {
				bidiDirMarkupServerName = "dir = \"" + globalization.obtainContextualDir(server.name) + "\"";
				bidiDirMarkupCluster = "dir = \"" + globalization.obtainContextualDir(cluster) + "\"";
				bidiDirMarkupHost = "dir = \"" + globalization.obtainContextualDir(server.host) + "\"";
				
				// TODO: special handling for user dir
				bidiDirMarkupUserDir = "dir = \"" + globalization.obtainContextualDir(server.wlpUserDir) + "\"";
				
			} else {
				bidiDirMarkupServerName = bidiDirMarkupCluster = bidiDirMarkupHost = bidiDirMarkupUserDir = "dir = \"" + dirValue + "\"";	
			}			
		}

		// Create server entry
		var spanInsert = "<span class=\"sr-only\">" + stringUtils.formatString(editorMessages.SERVER_TABLE_CELL_FOR_SCREEN_READER, [cluster, server.host, server.wlpUserDir]) + "</span>";
		var serverEntry = $("<tr tabindex=\"0\" class=\"serverExplorerServerEntry\" >" +
							"	<td><span " + bidiDirMarkupServerName + ">" + (searchFilter.length > 0? markSearchFilter(server.name, searchFilter) : server.name) + "</span>" + spanInsert +"</td>" +
							"	<td><span " + bidiDirMarkupCluster + ">" + (searchFilter.length > 0? markSearchFilter(cluster, searchFilter) : cluster) + "</span></td>" +
							"	<td><span " + bidiDirMarkupHost + ">" + (searchFilter.length > 0? markSearchFilter(server.host, searchFilter) : server.host) + "</span></td>" +
							"	<td><span " + bidiDirMarkupUserDir + ">" + (searchFilter.length > 0? markSearchFilter(server.wlpUserDir, searchFilter) : server.wlpUserDir) + "</span></td>" +
							"</tr>");
		
		// Associate server ID with entry
		serverEntry.data("serverId", server.id);
		
		// Add entry to table
		serverExplorerTableBody.append(serverEntry);
	};
	
	
	var markSearchFilter = function(value, searchFilter) {
		
		// Initialize value
		var markedValue = value;
		
		// Enclose each occurence of searchFilter in value in mark tags
		var index = value.indexOf(searchFilter);
		while(index != -1) {
			markedValue = value.substring(0, index) + "<mark>" + searchFilter + "</mark>" + value.substring(index + searchFilter.length);
			index = value.indexOf(searchFilter, index + 1);
		}
		return markedValue;
	};
	
	
	var handleSort = function(key, descending) {
		cachedFilteredServerList.sort(function(a, b) {
			var valueA = a[key];
			var valueB = b[key];
			if(!valueA) {
				valueA = "";
			}
			if(!valueB) {
				valueB = "";
			}
			if(descending) {
				return (valueA < valueB)? 1 : (valueA > valueB)? -1 : 0;
			} else {				
				return (valueA > valueB)? 1 : (valueA < valueB)? -1 : 0;
			}
		});
	};
	
	
	$(document).ready(function() {

		// Handle header outline in webkit (mouse)
		$("#serverExplorerTable").on("mousedown", function(event) {
			if("WebkitAppearance" in document.documentElement.style) {
				$("#serverExplorerTable").addClass("noOutline");
			}
		});
		
		
		// Handle header outline in webkit (keyboard)
		$("#serverExplorerTable").on("keydown", function(event) {
			if(event.keyCode != 16 && event.keyCode != 17 && event.keyCode != 18 && "WebkitAppearance" in document.documentElement.style) {
				$("#serverExplorerTable").removeClass("noOutline");
			}
		});
		
		
		// Handle sort
		$("#serverExplorer").on("vclick", ".columnHeader", function(event) {
			event.preventDefault();
			var header = $(this);
			if(header.hasClass("ascending") || header.hasClass("descending")) {
				header.toggleClass("ascending descending");
			} else {
				header.addClass("ascending");
			}
			$("#serverExplorer .columnHeader").not(this).removeClass("ascending descending");
			
			handleSort($(this).data("key"), header.hasClass("descending"));
			renderServerList();
		});

		
		// Handle server selection
		$("#serverExplorerTableBody").on("click", "tr.serverExplorerServerEntry", function(event) {
			event.preventDefault();
			var serverId = $(event.currentTarget).data("serverId");
			core.setServer(serverId);
		});
		
		
		$("#serverExplorerTableBody").on("keydown", "tr", function(event) {
			if(event.keyCode == 13) {
				event.preventDefault();
				$(event.currentTarget).trigger("click");
			}
		});
		
		
		// Handle back to server list
		$("#navbarChangeServerButton").on("vclick", function(event) {
			event.preventDefault();
			core.clearServer();
		});
		
		
		// Handle enter key on search field
		$("#serverExplorerSearchInput").on("keydown", function(event) {
			if(event.keyCode == 13) {
				event.preventDefault();
				var serverExplorerSearchButton = $("#serverExplorerSearchButton");
				if(serverExplorerSearchButton.attr("disabled") != "disabled") {
					serverExplorerSearchButton.trigger("click");
				}
			}
		});

		
		// Handle enabling of search button depending on value in search field
		$("#serverExplorerSearchInput").on("input", function(event) {
			
			// Handle bidi
			if(globalization.isBidiEnabled() && globalization.getBidiTextDirection() == "contextual") {
				$(this).attr("dir", globalization.obtainContextualDir(event.currentTarget.value));
			}
			
			var serverExplorerSearchButton = $("#serverExplorerSearchButton");
			if($(this).val().length > 0) {
				serverExplorerSearchButton.removeAttr("disabled");
				serverExplorerSearchButton.removeAttr("tabIndex");
				core.showControlById("serverExplorerClearButton");
			} else if(!filterApplied) {
				serverExplorerSearchButton.attr("disabled", "disabled");
				serverExplorerSearchButton.attr("tabIndex", "-1");
				core.hideControlById("serverExplorerClearButton");
			}
		});
		
		
		// Handle search button
		$("#serverExplorerSearchButton").on("vclick", function(event) {
			event.preventDefault();
			applySearchFilter();
			renderServerList();
			
			// Update clear and search buttons
			if($("#serverExplorerSearchInput").val().length == 0) {
				$("#serverExplorerClearButton").addClass("hidden");
				$(this).attr("disabled", "disabled");
			}
		});
		
		
		// Handle clear search button
		$("#serverExplorerClearButton").on("vclick", function(event) {
			event.preventDefault();
			$("#serverExplorerSearchInput").val("");
			$("#serverExplorerSearchButton").trigger("click");
		});
	
	});


	return {
		renderServerExplorer: renderServerExplorer
	};	
	
})();
